package net.jieyuu.mq;

import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import net.jieyuu.model.OrderMessage;
import net.jieyuu.service.ProductOrderService;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;


@Slf4j
@Component
@RabbitListener(queues = "${mqconfig.order_close_queue}")
public class ProductOrderMQListener {


    @Autowired
    private ProductOrderService productOrderService;

    /**
     * 消息重复消费，幂等性保障
     *
     * @param orderMessage
     * @param message
     * @param channel
     * @throws IOException
     */
    @RabbitHandler
    public void closeProductOrder(OrderMessage orderMessage, Message message, Channel channel) throws IOException {
        log.info("监听到消息:closeProductOrder:{} ", orderMessage);
        long msgTag = message.getMessageProperties().getDeliveryTag();

        try {
            boolean flag = productOrderService.closeProductOrder(orderMessage);
            if (flag) {// 关单成功
                channel.basicAck(msgTag, false);
            } else {// 失败
                channel.basicAck(msgTag, true);// 重新入队
            }
        } catch (IOException e) {
            log.error("定时关单失败:{}", orderMessage);
            channel.basicReject(msgTag, true);
        }
    }
}
