package net.jieyuu.mq;

import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import net.jieyuu.model.ProductMessage;
import net.jieyuu.service.ProductService;
import org.redisson.api.RedissonClient;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

@Slf4j
@Component
@RabbitListener(queues = "${mqconfig.stock_release_queue}")
public class ProductStockMQListener {

    @Autowired
    private ProductService productService;
    @Autowired
    private RedissonClient redissonClient;


    @RabbitHandler
    public void releaseProductStock(ProductMessage productMessage, Message message, Channel channel) throws IOException {

        log.info("监听到消息:releaseProductStock消息内容:{}", productMessage);
        long msgTag = message.getMessageProperties().getDeliveryTag();
        boolean flag = productService.releaseProductStock(productMessage);
        // 防止同时解锁多任务并发进入
        // 串行消费不需要加锁

        try {
            if (flag) {
                // 确认消息消费成功 不需要重新入队
                channel.basicAck(msgTag, false);
            } else {
                log.error("释放商品库存异常 flag=false,{}", productMessage);
                // 选择重新入队
                channel.basicReject(msgTag, true);
            }
        } catch (IOException e) {
            log.error("释放商品库存异常:{},msg:{}", e, productMessage);
            // 选择重新入队
            // 再发生问题就往外抛
            channel.basicReject(msgTag, true);
        }
    }


    @RabbitHandler
    public void releaseCouponRecord2(String msg, Message message, Channel channel) throws IOException {
        log.info(msg);
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), true);

    }
}
